#!/usr/bin/python3
# coding:utf-8

# Copyright (c) 2019 ISB CORPORATION <http://www.isb.co.jp/>
# All rights reserved.

import sys
import os
import signal
import time
import json
import isb_wsn_serial
import random

from time import sleep
from isb_wsn_serial import WiSunSerial


################################################################################
# Wi-SUN Dual Stack
################################################################################

def main(argDev):
	"""Wi-SUNメイン処理
	"""
	# 設定をjsonから読み込む
	ttyDev, init_settings, panc_addr = load_config()

	# シリアルポートオープン
	if(argDev is not None):
		ttyDev = argDev
	wsn_ser = WiSunSerial(ttyDev)
	if(wsn_ser.isOpen() == False): return 1

	# ハードウェアリセット
	wsn_ser.cmd_com_reset()
	# MACアドレス取得
	wsn_ser.cmd_com_get_addr()
	# 初期設定
	wsn_ser.cmd_com_init(init_settings)

	# PANコーディネータのIPv6アドレス
	ipv6 = isb_wsn_serial.mac2ipv6(panc_addr)

	while(True):
		# HAN動作開始  設定ファイルのPANコーディネーターに対して通常接続を行う
		while(True):
			ret = wsn_ser.cmd_han_start( 0x0000, panc_addr )
			if( ret == 0):
				break

		# データ送信要求 Unicast
		while(True):
			payload = b'\x12\x34\x56'
			ret = wsn_ser.cmd_com_send_data(
				ipv6, 1111,1111, payload)
			if(ret == 0):
				print("UDPデータ送信:Success")
				break

		# HAN動作終了
		wsn_ser.cmd_han_end()

		# Deep Sleep
		while(True):
			ret = wsn_ser.cmd_han_deepsleep()
			if(ret == 0):
				break
			else:
				sleep(0.1)
		
		# 待機
		sleep(60)

		# Wake Up
		while(True):
			ret = wsn_ser.cmd_han_wakeup()
			if(ret == 0):
				break

	return 0

################################################################################
# Application Config
################################################################################

def load_config():
	"""設定をjsonファイルから読み込む
	"""
	f = open("ed_config_1.json", 'r')
	jsonObj = json.load(f)
	f.close()

	# シリアルポート
	ttyDev = jsonObj["SerialInterface"]["port"]
	# 動作モード
	mode = jsonObj["InitialSetting"]["actionMode"]
	# HAN Sleep機能
	if( jsonObj["InitialSetting"]["sleepSupport"] ):
		sleepSupport = 0x01
	else:
		sleepSupport = 0x00
	# チャネル
	channel = jsonObj["InitialSetting"]["channel"]
	# 送信出力
	txPower = jsonObj["InitialSetting"]["txpower"]

	# 初期設定
	init_settings = [
		mode,			# 動作モード(0x01:PANコーディネータ)
		sleepSupport,	# HAN Sleep 機能設定(0x00:無効,0x01:有効)
		channel,		# チャネル(0x04～0x11)
		txPower			# 送信電力(0x00:20mW,0x01:10mW,0x02:1mW)
	]

	# 接続先のPANコーディネータ
	panc_addr = bytes.fromhex(jsonObj["HAN"]["pancAddr"])

	return ttyDev, init_settings, panc_addr

################################################################################

def sigint_handler(signal, frame):
	print('\nterminated by user')
	sys.exit(0)

if __name__ == '__main__':
	signal.signal(signal.SIGINT, sigint_handler)

	import argparse
	import sys,codecs
	sys.stdout = codecs.getwriter("utf-8")(sys.stdout.detach())

	parser = argparse.ArgumentParser( description='Sample script' )
	parser.add_argument(
		'-t','--ttydev',
		dest='ttydev', default=None, required=False,
		help='specify tty device.\n ex: Win=\'COMx\', Linux=\'/dev/ttyUSBx\'' )
	args = parser.parse_args()

	ret = main(args.ttydev)
	sys.exit(ret)
