#!/usr/bin/python3
# coding:utf-8

# Copyright (c) 2019 ISB CORPORATION <http://www.isb.co.jp/>
# All rights reserved.

import sys
import os
import signal
import json
import time
from time import sleep
import threading
import queue

import isb_wsn_serial
from isb_wsn_serial import WiSunSerial

################################################################################
# UARTコマンド送信スレッド
################################################################################
_gSendQueue = queue.Queue()
"""シリアルへのコマンド送信キュー
"""
def send_queue_get():
	global _gSendQueue
	return _gSendQueue.get(block=True, timeout=None)

def send_queue_put(data):
	global _gSendQueue
	return _gSendQueue.put(data)

_gLock = threading.Lock()
"""コマンドの二重要求抑止用ロック
"""
def acquire_lock():
	global _gLock
	isLocked = _gLock.acquire(blocking=True, timeout=5)
	if(not isLocked):
		print("lock timeout.")

	return 

def release_lock():
	global _gLock
	_gLock.acquire(blocking=False)
	_gLock.release()
	return

def thread_send(serial):
	"""Wi-SUNスタック UARTコマンド送信スレッド
	"""
	while True:
		command = send_queue_get()
		if(command is None):
			continue

		# 要求コマンド送信前にロックを取得
		acquire_lock()
		ret = serial.send(command)
		if( ret != 0):
			# 送信失敗時は即ロックを解除
			release_lock()

	return 0

################################################################################
# UARTコマンド受信スレッド
################################################################################
def thread_recv(serial):
	"""Wi-SUNスタック UARTコマンド受信スレッド
	"""
	while(True):
		# シリアルデータの受信
		recvData = serial.recv()
		if(len(recvData) == 0):
			continue
		
		cmdCode = isb_wsn_serial.get_cmd_code(recvData)
		cmdData = isb_wsn_serial.get_cmd_data(recvData)
		if( (cmdCode & 0x6000) == 0x2000 ): # 応答コマンド
			if(cmdData[0] != 0x3D):
				# 要求処理中の要求でなければロックを解除
				release_lock()

		# 各コマンドを受信したときの処理
		if(cmdCode == 0x601A): # 接続状態変更通知
			state = cmdData[0]
			mac = cmdData[1:9]
			strMac = "".join(format(x, "02X")for x in mac)
			rssi = cmdData[9] - 256
			
			if (state == 0x01): # MAC接続完了
				print("[CMD]接続状態変更通知 : MAC接続完了 \'{:s}\' RSSI={:d}".format(strMac,rssi))
			elif (state == 0x03): # MAC切断
				print("[CMD]接続状態変更通知 : MAC切断 \'{:s}\'".format(strMac))
			
		elif(cmdCode == 0x6018): # データ受信通知
			# 送信元のIPv6アドレスをMACアドレスに変換
			srcAddr = isb_wsn_serial.mac_from_ipv6(cmdData[0:16])
			rssi = int.from_bytes([ cmdData[24] ], 'big', signed=True)
			
			# 送信先ポート
			destPort = int.from_bytes(cmdData[18:20], byteorder='big')

			# ペイロード
			payload = cmdData[27:]
			strPayload = "".join(format(x, "02X")for x in payload)

			print("[CMD]データ受信通知 : Received from {:s}, RSSI={:d}, destPort={:d}".format(
				srcAddr, rssi, destPort))
			print(" Data({:d}byte)={:s}".format(len(payload),strPayload))

			if(destPort == 1111) and (payload == b'\x12\x34\x56'):
				# HANデバイスリスト削除コマンドを発行
				sendData = isb_wsn_serial.build_req_cmd(0x006A, bytes.fromhex(srcAddr) )
				send_queue_put( sendData )

		elif(cmdCode == 0x206A): # HANデバイス削除 応答
			if(cmdData[0] != isb_wsn_serial.COMMAND_SUCCESS):
				print("[CMD]HANデバイスリスト削除 : Failure")
			else:
				print("[CMD]HANデバイスリスト削除 : Success")

		else:
			continue
		
	return 0

################################################################################
# Application Config
################################################################################

def load_config():
	"""設定をjsonファイルから読み込む
	"""
	basedir = os.path.dirname(os.path.abspath(__file__))
	f = open(basedir+"/pc_config.json", 'r')
	jsonObj = json.load(f)
	f.close()

	# シリアルポート
	ttyDev = jsonObj["SerialInterface"]["port"]
	# 動作モード
	mode = jsonObj["InitialSetting"]["actionMode"]
	# HAN Sleep機能
	if( jsonObj["InitialSetting"]["sleepSupport"] ):
		sleepSupport = 0x01
	else:
		sleepSupport = 0x00
	# チャネル
	channel = jsonObj["InitialSetting"]["channel"]
	# 送信出力
	txPower = jsonObj["InitialSetting"]["txpower"]
	# PAN-ID
	panid = int(jsonObj["InitialSetting"]["panid"], 16)

	# 初期設定
	init_settings = [
		mode,			# 動作モード(0x01:PANコーディネータ,0x05:Dual)
		sleepSupport,	# HAN Sleep 機能設定(0x00:無効,0x01:有効)
		channel,		# チャネル(0x04～0x11)
		txPower			# 送信電力(0x00:20mW,0x01:10mW,0x02:1mW)
	]

	return ttyDev, init_settings, panid


################################################################################
# Application Main
################################################################################
def main(argDev):
	# 設定をjsonから読み込む 
	ttyDev, init_settings, panid = load_config()
	# シリアルポートオープン
	if(argDev is not None):
		ttyDev = argDev
	wsn_ser = WiSunSerial(ttyDev)
	if(wsn_ser.isOpen() == False): return 1

	# ハードウェアリセット
	wsn_ser.cmd_com_reset()
	# MACアドレス取得
	wsn_ser.cmd_com_get_addr()
	# 初期設定
	wsn_ser.cmd_com_init(init_settings)
	# HAN動作開始
	wsn_ser.cmd_han_start( panid )
	# UDPポートのオープン
	wsn_ser.cmd_com_open_port(1111)

	# 受信制御スレッド起動
	recv_t = threading.Thread(target=thread_recv, args=(wsn_ser, ))
	recv_t.start()

	# 送信制御スレッド起動
	send_t = threading.Thread(target=thread_send, args=(wsn_ser, ))
	send_t.start()

	# 受信制御スレッドの終了を待ち受ける
	recv_t.join()


def sigint_handler(signal, frame):
	print('\nterminated by user')
	sys.exit(0)

if __name__ == '__main__':
	signal.signal(signal.SIGINT, sigint_handler)

	import sys,codecs
	sys.stdout = codecs.getwriter("utf-8")(sys.stdout.detach())

	import argparse
	parser = argparse.ArgumentParser( description='Sample script' )
	parser.add_argument(
		'-t','--ttydev',
		dest='ttydev', default=None, required=False,
		help='specify tty device.\n ex: Win=\'COMx\', Linux=\'/dev/ttyUSBx\'' )
	args = parser.parse_args()

	ret = main(args.ttydev)
	sys.exit(ret)

